import Queue

from udd import (
    icommon,
    iconfig,
    idb,
    tests,
    )

from udd.scripts import mass_import


def get_logger(test):
    """Get a logger ensuring it's cleaned up after the test ends.

    Loggers used in tests should be cleaned up after use or they leak from test
    to test leading to confusing test failures.
    """
    logger = mass_import.create_logger()
    for h in logger.handlers:
        test.addCleanup(logger.removeHandler, h)
    return logger


class TestAllQueue(tests.TestCaseWithConfig):

    def test_empty(self):
        q = mass_import.AllQueue(get_logger(self))
        self.assertRaises(Queue.Empty, q.get_nowait)


class TestQueueFromFile(tests.TestCaseWithConfig):

    import_file_path = 'list'

    def _set_store_content(self, store, content=None):
        if content is None:
            content = ''
        # We need to make sure we never try to push
        content += 'pi.explicit_imports = %s\n' % (self.import_file_path,)
        return super(TestQueueFromFile,
                     self)._set_store_content(store, content)

    def get_queue(self, content):
        with open(self.import_file_path, 'w') as f:
            f.write(content)
        q = mass_import.QueueFromFile(get_logger(self))
        return q

    def test_empty_file(self):
        q = self.get_queue('')
        self.assertRaises(Queue.Empty, q.get_nowait)

    def test_single_package(self):
        q = self.get_queue('foo')
        self.assertEquals((0, 'foo'), q.next_job())
        self.assertRaises(Queue.Empty, q.next_job)

    def test_various_lines(self):
        q = self.get_queue('foo\n\nbar # and a comment\n# only comment\n'
                           '#first#second comment\n')
        self.assertEquals((0, 'foo'), q.next_job())
        self.assertEquals((0, 'bar'), q.next_job())
        self.assertRaises(Queue.Empty, q.next_job)


class TestImporter(tests.TestCaseWithConfig):

    def test_expanded_import_command(self):
        logger = get_logger(self)
        imp = mass_import.Importer(logger, 'foo', '12',
                                   _command='import-package {package}')
        self.assertEquals(['import-package', 'foo'], imp.args)


class TestImportController(tests.TestCaseWithConfig):

    def _set_store_content(self, store, content=None):
        if content is None:
            content = ''
        # We need to make sure we never try to push
        content += ('pi.imp_args = --no-push\n'
                    'pi.import_command = {pi.install_dir}/bin/import-package'
                    ' {pi.imp_args} {package}\n'
                    )
        return super(TestImportController,
                     self)._set_store_content(store, content)

    def test_empty_does_nothing(self):
        logger = get_logger(self)
        import_controller = mass_import.ImportController(logger)
        # Sanity check: ensure the package db is empty
        conf = iconfig.ImporterStack()
        packages_db = icommon.PackageDatabase(
            *idb.get_db_connection(conf, 'package'))
        self.assertRaises(StopIteration, packages_db.get_one, [])
        # Run the import controller, there should be nothing to import
        import_controller.run()
        self.assertTrue(import_controller.driver.stopped.isSet())
        self.assertTrue(import_controller.driver.queue_closed.isSet())


class TestImportControllerFailuresOnly(tests.TestCaseWithConfig):

    def _set_store_content(self, store, content=None):
        if content is None:
            content = ''
        # We need to make sure we never try to push
        content += 'pi.retry_all_failed_jobs = true\n'
        return super(TestImportControllerFailuresOnly,
                     self)._set_store_content(store, content)

    def test_empty_does_nothing(self):
        logger = get_logger(self)
        import_controller = mass_import.ImportController(logger)
        # Run the import controller, there should be nothing to import
        import_controller.run()
        self.assertTrue(import_controller.driver.stopped.isSet())
        self.assertTrue(import_controller.driver.queue_closed.isSet())


class TestImportControllerFromFile(tests.TestCaseWithConfig):

    import_file_path = 'list'

    def _set_store_content(self, store, content=None):
        if content is None:
            content = ''
        # We need to make sure we never try to push
        content += 'pi.explicit_imports = %s\n' % (self.import_file_path,)
        return super(TestImportControllerFromFile,
                     self)._set_store_content(store, content)

    def test_empty_does_nothing(self):
        with open(self.import_file_path, 'w') as f:
            f.write('')
        logger = get_logger(self)
        import_controller = mass_import.ImportController(logger)
        # Run the import controller, there should be nothing to import
        import_controller.run()
        self.assertTrue(import_controller.driver.stopped.isSet())
        self.assertTrue(import_controller.driver.queue_closed.isSet())
        self.assertEquals(mass_import.QueueFromFile,
                          import_controller.driver.queue_class)

